@@ -1,3 +1,9 @@
+// ****************************************************************
+// Copyright 2007, Charlie Poole
+// This is free software licensed under the NUnit license. You may
+// obtain a copy of the license at http://nunit.org/?p=license&r=2.4
+// ****************************************************************
+
 using System;
 using NUnit.Util;
 
@@ -45,7 +51,7 @@ namespace NUnit.UiKit
 					Console.WriteLine( "Exception:" );
 					Console.WriteLine( ex );
 					//throw new TestEventInvocationException( ex );
-					throw ex;
+					//throw;
 				}
 			}
 		}
