@@ -1,9 +1,16 @@
+// ****************************************************************
+// This is free software licensed under the NUnit license. You
+// may obtain a copy of the license as well as information regarding
+// copyright ownership at http://nunit.org/?p=license&r=2.4.
+// ****************************************************************
+
 using System;
 using System.Collections;
 using System.ComponentModel;
 using System.Drawing;
 using System.Data;
 using System.Windows.Forms;
+using NUnit.Core;
 using NUnit.Util;
 
 namespace NUnit.UiKit
@@ -41,19 +48,17 @@ namespace NUnit.UiKit
 		private System.Windows.Forms.Button removeCategory;
 		private System.Windows.Forms.Button clearAllButton;
 		private System.Windows.Forms.Button checkFailedButton;
-		private System.Windows.Forms.MenuItem viewMenu;
+		private System.Windows.Forms.MenuItem treeMenu;
+		private System.Windows.Forms.MenuItem checkBoxesMenuItem;
+		private System.Windows.Forms.MenuItem treeMenuSeparatorItem1;
 		private System.Windows.Forms.MenuItem expandMenuItem;
 		private System.Windows.Forms.MenuItem collapseMenuItem;
-		private System.Windows.Forms.MenuItem viewMenuSeparatorItem1;
+		private System.Windows.Forms.MenuItem treeMenuSeparatorItem2;
 		private System.Windows.Forms.MenuItem expandAllMenuItem;
 		private System.Windows.Forms.MenuItem collapseAllMenuItem;
-		private System.Windows.Forms.MenuItem viewMenuSeparatorItem2;
-		private System.Windows.Forms.MenuItem expandFixturesMenuItem;
-		private System.Windows.Forms.MenuItem collapseFixturesMenuItem;
-		private System.Windows.Forms.MenuItem viewMenuSeparatorItem3;
+		private System.Windows.Forms.MenuItem hideTestsMenuItem;
+		private System.Windows.Forms.MenuItem treeMenuSeparatorItem3;
 		private System.Windows.Forms.MenuItem propertiesMenuItem;
-		private System.Windows.Forms.MenuItem checkBoxesMenuItem;
-		private System.Windows.Forms.MenuItem menuItem2;
 		private System.Windows.Forms.CheckBox excludeCheckbox;
 
 		/// <summary> 
@@ -65,9 +70,9 @@ namespace NUnit.UiKit
 
 		#region Properties
 
-		public MenuItem ViewMenu 
+		public MenuItem TreeMenu 
 		{
-			get { return viewMenu; }
+			get { return treeMenu; }
 		}
 
 		public string[] SelectedCategories
@@ -82,6 +87,7 @@ namespace NUnit.UiKit
 			}
 		}
 
+		[Browsable(false)]
 		public bool ShowCheckBoxes
 		{
 			get { return tests.CheckBoxes; }
@@ -94,25 +100,6 @@ namespace NUnit.UiKit
 				checkBoxesMenuItem.Checked = value;
 			}
 		}
-
-		public bool VisualStudioSupport
-		{
-			get { return tests.VisualStudioSupport; }
-			set { tests.VisualStudioSupport = value; }
-		}
-
-		public TestSuiteTreeView.DisplayStyle InitialDisplay
-		{
-			get { return tests.InitialDisplay; }
-			set { tests.InitialDisplay = value; }
-		}
-
-		public bool ClearResultsOnChange
-		{
-			get { return tests.ClearResultsOnChange; }
-			set { tests.ClearResultsOnChange = value; }
-		}
-
 		#endregion
 
 		#region Construction and Initialization
@@ -121,109 +108,97 @@ namespace NUnit.UiKit
 		{
 			// This call is required by the Windows.Forms Form Designer.
 			InitializeComponent();
-			viewMenu = new MenuItem();
+			treeMenu = new MenuItem();
+			this.checkBoxesMenuItem = new System.Windows.Forms.MenuItem();
+			this.treeMenuSeparatorItem1 = new System.Windows.Forms.MenuItem();
 			this.expandMenuItem = new System.Windows.Forms.MenuItem();
 			this.collapseMenuItem = new System.Windows.Forms.MenuItem();
-			this.viewMenuSeparatorItem1 = new System.Windows.Forms.MenuItem();
+			this.treeMenuSeparatorItem2 = new System.Windows.Forms.MenuItem();
 			this.expandAllMenuItem = new System.Windows.Forms.MenuItem();
 			this.collapseAllMenuItem = new System.Windows.Forms.MenuItem();
-			this.viewMenuSeparatorItem2 = new System.Windows.Forms.MenuItem();
-			this.expandFixturesMenuItem = new System.Windows.Forms.MenuItem();
-			this.collapseFixturesMenuItem = new System.Windows.Forms.MenuItem();
-			this.viewMenuSeparatorItem3 = new System.Windows.Forms.MenuItem();
+			this.hideTestsMenuItem = new System.Windows.Forms.MenuItem();
+			this.treeMenuSeparatorItem3 = new System.Windows.Forms.MenuItem();
 			this.propertiesMenuItem = new System.Windows.Forms.MenuItem();
-			this.checkBoxesMenuItem = new System.Windows.Forms.MenuItem();
-			this.menuItem2 = new System.Windows.Forms.MenuItem();
 
 			// 
-			// viewMenu
+			// treeMenu
 			// 
-			this.viewMenu.MergeType = MenuMerge.Add;
-			this.viewMenu.MergeOrder = 1;
-			this.viewMenu.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
-										this.expandMenuItem,
-										this.collapseMenuItem,
-										this.viewMenuSeparatorItem1,
-										this.expandAllMenuItem,
-										this.collapseAllMenuItem,
-										this.viewMenuSeparatorItem2,
-										this.expandFixturesMenuItem,
-										this.collapseFixturesMenuItem,
-										this.viewMenuSeparatorItem3,
-										this.propertiesMenuItem,
-										this.checkBoxesMenuItem,
-										this.menuItem2});
-			this.viewMenu.Text = "&View";
-			this.viewMenu.Visible = false;
-			this.viewMenu.Popup += new System.EventHandler(this.viewMenu_Popup);
+			this.treeMenu.MergeType = MenuMerge.Add;
+			this.treeMenu.MergeOrder = 1;
+			this.treeMenu.MenuItems.AddRange(
+				new System.Windows.Forms.MenuItem[] 
+				{
+					this.checkBoxesMenuItem,
+					this.treeMenuSeparatorItem1,
+					this.expandMenuItem,
+					this.collapseMenuItem,
+					this.treeMenuSeparatorItem2,
+					this.expandAllMenuItem,
+					this.collapseAllMenuItem,
+					this.hideTestsMenuItem,
+					this.treeMenuSeparatorItem3,
+					this.propertiesMenuItem 
+				} );
+			this.treeMenu.Text = "&Tree";
+			this.treeMenu.Visible = false;
+			this.treeMenu.Popup += new System.EventHandler(this.treeMenu_Popup);
+			// 
+			// checkBoxesMenuItem
+			// 
+			this.checkBoxesMenuItem.Index = 0;
+			this.checkBoxesMenuItem.Text = "Show Check&Boxes";
+			this.checkBoxesMenuItem.Click += new System.EventHandler(this.checkBoxesMenuItem_Click);
+			// 
+			// treeMenuSeparatorItem1
+			// 
+			this.treeMenuSeparatorItem1.Index = 1;
+			this.treeMenuSeparatorItem1.Text = "-";
 			// 
 			// expandMenuItem
 			// 
-			this.expandMenuItem.Index = 0;
+			this.expandMenuItem.Index = 2;
 			this.expandMenuItem.Text = "&Expand";
 			this.expandMenuItem.Click += new System.EventHandler(this.expandMenuItem_Click);
 			// 
 			// collapseMenuItem
 			// 
-			this.collapseMenuItem.Index = 1;
+			this.collapseMenuItem.Index = 3;
 			this.collapseMenuItem.Text = "&Collapse";
 			this.collapseMenuItem.Click += new System.EventHandler(this.collapseMenuItem_Click);
 			// 
-			// viewMenuSeparatorItem1
+			// treeMenuSeparatorItem2
 			// 
-			this.viewMenuSeparatorItem1.Index = 2;
-			this.viewMenuSeparatorItem1.Text = "-";
+			this.treeMenuSeparatorItem2.Index = 4;
+			this.treeMenuSeparatorItem2.Text = "-";
 			// 
 			// expandAllMenuItem
 			// 
-			this.expandAllMenuItem.Index = 3;
+			this.expandAllMenuItem.Index = 5;
 			this.expandAllMenuItem.Text = "Expand All";
 			this.expandAllMenuItem.Click += new System.EventHandler(this.expandAllMenuItem_Click);
 			// 
 			// collapseAllMenuItem
 			// 
-			this.collapseAllMenuItem.Index = 4;
+			this.collapseAllMenuItem.Index = 6;
 			this.collapseAllMenuItem.Text = "Collapse All";
 			this.collapseAllMenuItem.Click += new System.EventHandler(this.collapseAllMenuItem_Click);
 			// 
-			// viewMenuSeparatorItem2
+			// hideTestsMenuItem
 			// 
-			this.viewMenuSeparatorItem2.Index = 5;
-			this.viewMenuSeparatorItem2.Text = "-";
+			this.hideTestsMenuItem.Index = 7;
+			this.hideTestsMenuItem.Text = "Hide Tests";
+			this.hideTestsMenuItem.Click += new System.EventHandler(this.hideTestsMenuItem_Click);
 			// 
-			// expandFixturesMenuItem
+			// treeMenuSeparatorItem3
 			// 
-			this.expandFixturesMenuItem.Index = 6;
-			this.expandFixturesMenuItem.Text = "Expand Fixtures";
-			this.expandFixturesMenuItem.Click += new System.EventHandler(this.expandFixturesMenuItem_Click);
-			// 
-			// collapseFixturesMenuItem
-			// 
-			this.collapseFixturesMenuItem.Index = 7;
-			this.collapseFixturesMenuItem.Text = "Collapse Fixtures";
-			this.collapseFixturesMenuItem.Click += new System.EventHandler(this.collapseFixturesMenuItem_Click);
-			// 
-			// viewMenuSeparatorItem3
-			// 
-			this.viewMenuSeparatorItem3.Index = 8;
-			this.viewMenuSeparatorItem3.Text = "-";
+			this.treeMenuSeparatorItem3.Index = 8;
+			this.treeMenuSeparatorItem3.Text = "-";
 			// 
 			// propertiesMenuItem
 			// 
 			this.propertiesMenuItem.Index = 9;
 			this.propertiesMenuItem.Text = "&Properties";
 			this.propertiesMenuItem.Click += new System.EventHandler(this.propertiesMenuItem_Click);
-			// 
-			// checkBoxesMenuItem
-			// 
-			this.checkBoxesMenuItem.Index = 0;
-			this.checkBoxesMenuItem.Text = "Check&Boxes";
-			this.checkBoxesMenuItem.Click += new System.EventHandler(this.checkBoxesMenuItem_Click);
-			// 
-			// menuItem2
-			// 
-			this.menuItem2.Index = 1;
-			this.menuItem2.Text = "-";
 
 
 			tests.SelectedTestChanged += new SelectedTestChangedHandler(tests_SelectedTestChanged);
@@ -232,13 +207,44 @@ namespace NUnit.UiKit
 			this.excludeCheckbox.Enabled = false;
 		}
 
+		protected override void OnLoad(EventArgs e)
+		{
+			if ( !this.DesignMode )
+			{
+				this.ShowCheckBoxes = 
+					Services.UserSettings.GetSetting( "Options.ShowCheckBoxes", false );
+				Initialize( Services.TestLoader );
+				Services.UserSettings.Changed += new SettingsEventHandler(UserSettings_Changed);
+			}
+
+			base.OnLoad (e);
+		}
+
 		public void Initialize(TestLoader loader) 
 		{
 			this.tests.Initialize(loader, loader.Events);
 			this.loader = loader;
 			loader.Events.TestLoaded += new NUnit.Util.TestEventHandler(events_TestLoaded);
 			loader.Events.TestReloaded += new NUnit.Util.TestEventHandler(events_TestReloaded);
-			loader.Events.TestUnloaded += new NUnit.Util.TestEventHandler(Events_TestUnloaded);
+			loader.Events.TestUnloaded += new NUnit.Util.TestEventHandler(events_TestUnloaded);
+		}
+
+		public void SelectCategories( string[] categories, bool exclude )
+		{
+			foreach( string category in categories )
+			{
+				if ( availableCategories.Contains( category ) )
+				{
+					selectedList.Items.Add( category );
+					availableList.Items.Remove( category );
+				}
+
+				this.excludeCheckbox.Checked = exclude;
+			}
+
+			UpdateCategoryFilter();
+			if (this.SelectedCategories.Length > 0)
+				this.excludeCheckbox.Enabled = true;
 		}
 
 		/// <summary> 
@@ -260,7 +266,7 @@ namespace NUnit.UiKit
 
 		#region View Menu Handlers
 
-		private void viewMenu_Popup(object sender, System.EventArgs e)
+		private void treeMenu_Popup(object sender, System.EventArgs e)
 		{
 			TreeNode selectedNode = tests.SelectedNode;
 			if ( selectedNode != null && selectedNode.Nodes.Count > 0 )
@@ -287,26 +293,25 @@ namespace NUnit.UiKit
 
 		private void collapseAllMenuItem_Click(object sender, System.EventArgs e)
 		{
+			tests.BeginUpdate();
 			tests.CollapseAll();
+			tests.EndUpdate();
 
 			// Compensate for a bug in the underlying control
 			if ( tests.Nodes.Count > 0 )
 				tests.SelectedNode = tests.Nodes[0];	
 		}
 
+		private void hideTestsMenuItem_Click(object sender, System.EventArgs e)
+		{
+			tests.HideTests();
+		}
+
 		private void expandAllMenuItem_Click(object sender, System.EventArgs e)
 		{
+			tests.BeginUpdate();
 			tests.ExpandAll();
-		}
-
-		private void collapseFixturesMenuItem_Click(object sender, System.EventArgs e)
-		{
-			tests.CollapseFixtures();		
-		}
-
-		private void expandFixturesMenuItem_Click(object sender, System.EventArgs e)
-		{
-			tests.ExpandFixtures();		
+			tests.EndUpdate();
 		}
 
 		private void propertiesMenuItem_Click(object sender, System.EventArgs e)
@@ -324,6 +329,7 @@ namespace NUnit.UiKit
 		/// </summary>
 		private void InitializeComponent()
 		{
+			System.Resources.ResourceManager resources = new System.Resources.ResourceManager(typeof(TestTree));
 			this.tabs = new System.Windows.Forms.TabControl();
 			this.testPage = new System.Windows.Forms.TabPage();
 			this.testPanel = new System.Windows.Forms.Panel();
@@ -356,10 +362,12 @@ namespace NUnit.UiKit
 			// 
 			// tabs
 			// 
+			this.tabs.Alignment = System.Windows.Forms.TabAlignment.Left;
 			this.tabs.Controls.Add(this.testPage);
 			this.tabs.Controls.Add(this.categoryPage);
 			this.tabs.Dock = System.Windows.Forms.DockStyle.Fill;
 			this.tabs.Location = new System.Drawing.Point(0, 0);
+			this.tabs.Multiline = true;
 			this.tabs.Name = "tabs";
 			this.tabs.SelectedIndex = 0;
 			this.tabs.Size = new System.Drawing.Size(248, 496);
@@ -368,9 +376,9 @@ namespace NUnit.UiKit
 			// testPage
 			// 
 			this.testPage.Controls.Add(this.testPanel);
-			this.testPage.Location = new System.Drawing.Point(4, 25);
+			this.testPage.Location = new System.Drawing.Point(25, 4);
 			this.testPage.Name = "testPage";
-			this.testPage.Size = new System.Drawing.Size(240, 467);
+			this.testPage.Size = new System.Drawing.Size(219, 488);
 			this.testPage.TabIndex = 0;
 			this.testPage.Text = "Tests";
 			// 
@@ -381,7 +389,7 @@ namespace NUnit.UiKit
 			this.testPanel.Dock = System.Windows.Forms.DockStyle.Fill;
 			this.testPanel.Location = new System.Drawing.Point(0, 0);
 			this.testPanel.Name = "testPanel";
-			this.testPanel.Size = new System.Drawing.Size(240, 467);
+			this.testPanel.Size = new System.Drawing.Size(219, 488);
 			this.testPanel.TabIndex = 0;
 			// 
 			// treePanel
@@ -390,36 +398,34 @@ namespace NUnit.UiKit
 			this.treePanel.Dock = System.Windows.Forms.DockStyle.Fill;
 			this.treePanel.Location = new System.Drawing.Point(0, 0);
 			this.treePanel.Name = "treePanel";
-			this.treePanel.Size = new System.Drawing.Size(240, 427);
+			this.treePanel.Size = new System.Drawing.Size(219, 448);
 			this.treePanel.TabIndex = 0;
 			// 
 			// tests
 			// 
 			this.tests.AllowDrop = true;
-			this.tests.CheckBoxes = true;
 			this.tests.Dock = System.Windows.Forms.DockStyle.Fill;
-			this.tests.ExcludeSelectedCategories = false;
 			this.tests.HideSelection = false;
 			this.tests.Location = new System.Drawing.Point(0, 0);
 			this.tests.Name = "tests";
-			this.tests.SelectedCategories = null;
-			this.tests.Size = new System.Drawing.Size(240, 427);
+			this.tests.Size = new System.Drawing.Size(219, 448);
 			this.tests.TabIndex = 0;
+			this.tests.CheckBoxesChanged += new System.EventHandler(this.tests_CheckBoxesChanged);
 			// 
 			// buttonPanel
 			// 
 			this.buttonPanel.Controls.Add(this.checkFailedButton);
 			this.buttonPanel.Controls.Add(this.clearAllButton);
 			this.buttonPanel.Dock = System.Windows.Forms.DockStyle.Bottom;
-			this.buttonPanel.Location = new System.Drawing.Point(0, 427);
+			this.buttonPanel.Location = new System.Drawing.Point(0, 448);
 			this.buttonPanel.Name = "buttonPanel";
-			this.buttonPanel.Size = new System.Drawing.Size(240, 40);
+			this.buttonPanel.Size = new System.Drawing.Size(219, 40);
 			this.buttonPanel.TabIndex = 1;
 			// 
 			// checkFailedButton
 			// 
 			this.checkFailedButton.Anchor = System.Windows.Forms.AnchorStyles.Top;
-			this.checkFailedButton.Location = new System.Drawing.Point(128, 8);
+			this.checkFailedButton.Location = new System.Drawing.Point(117, 8);
 			this.checkFailedButton.Name = "checkFailedButton";
 			this.checkFailedButton.Size = new System.Drawing.Size(96, 23);
 			this.checkFailedButton.TabIndex = 1;
@@ -429,7 +435,7 @@ namespace NUnit.UiKit
 			// clearAllButton
 			// 
 			this.clearAllButton.Anchor = System.Windows.Forms.AnchorStyles.Top;
-			this.clearAllButton.Location = new System.Drawing.Point(16, 8);
+			this.clearAllButton.Location = new System.Drawing.Point(13, 8);
 			this.clearAllButton.Name = "clearAllButton";
 			this.clearAllButton.Size = new System.Drawing.Size(96, 23);
 			this.clearAllButton.TabIndex = 0;
@@ -439,9 +445,9 @@ namespace NUnit.UiKit
 			// categoryPage
 			// 
 			this.categoryPage.Controls.Add(this.categoryPanel);
-			this.categoryPage.Location = new System.Drawing.Point(4, 25);
+			this.categoryPage.Location = new System.Drawing.Point(25, 4);
 			this.categoryPage.Name = "categoryPage";
-			this.categoryPage.Size = new System.Drawing.Size(240, 467);
+			this.categoryPage.Size = new System.Drawing.Size(219, 488);
 			this.categoryPage.TabIndex = 1;
 			this.categoryPage.Text = "Categories";
 			// 
@@ -453,7 +459,7 @@ namespace NUnit.UiKit
 			this.categoryPanel.Dock = System.Windows.Forms.DockStyle.Fill;
 			this.categoryPanel.Location = new System.Drawing.Point(0, 0);
 			this.categoryPanel.Name = "categoryPanel";
-			this.categoryPanel.Size = new System.Drawing.Size(240, 467);
+			this.categoryPanel.Size = new System.Drawing.Size(219, 488);
 			this.categoryPanel.TabIndex = 0;
 			// 
 			// categoryButtonPanel
@@ -462,15 +468,15 @@ namespace NUnit.UiKit
 				| System.Windows.Forms.AnchorStyles.Right)));
 			this.categoryButtonPanel.Controls.Add(this.removeCategory);
 			this.categoryButtonPanel.Controls.Add(this.addCategory);
-			this.categoryButtonPanel.Location = new System.Drawing.Point(8, 256);
+			this.categoryButtonPanel.Location = new System.Drawing.Point(8, 280);
 			this.categoryButtonPanel.Name = "categoryButtonPanel";
-			this.categoryButtonPanel.Size = new System.Drawing.Size(224, 40);
+			this.categoryButtonPanel.Size = new System.Drawing.Size(203, 40);
 			this.categoryButtonPanel.TabIndex = 1;
 			// 
 			// removeCategory
 			// 
 			this.removeCategory.Anchor = System.Windows.Forms.AnchorStyles.Top;
-			this.removeCategory.Location = new System.Drawing.Point(120, 8);
+			this.removeCategory.Location = new System.Drawing.Point(109, 8);
 			this.removeCategory.Name = "removeCategory";
 			this.removeCategory.TabIndex = 1;
 			this.removeCategory.Text = "Remove";
@@ -479,7 +485,7 @@ namespace NUnit.UiKit
 			// addCategory
 			// 
 			this.addCategory.Anchor = System.Windows.Forms.AnchorStyles.Top;
-			this.addCategory.Location = new System.Drawing.Point(32, 8);
+			this.addCategory.Location = new System.Drawing.Point(21, 8);
 			this.addCategory.Name = "addCategory";
 			this.addCategory.TabIndex = 0;
 			this.addCategory.Text = "Add";
@@ -491,9 +497,9 @@ namespace NUnit.UiKit
 				| System.Windows.Forms.AnchorStyles.Right)));
 			this.selectedCategories.Controls.Add(this.selectedList);
 			this.selectedCategories.Controls.Add(this.excludeCheckbox);
-			this.selectedCategories.Location = new System.Drawing.Point(8, 304);
+			this.selectedCategories.Location = new System.Drawing.Point(8, 328);
 			this.selectedCategories.Name = "selectedCategories";
-			this.selectedCategories.Size = new System.Drawing.Size(224, 152);
+			this.selectedCategories.Size = new System.Drawing.Size(203, 144);
 			this.selectedCategories.TabIndex = 2;
 			this.selectedCategories.TabStop = false;
 			this.selectedCategories.Text = "Selected Categories";
@@ -503,10 +509,11 @@ namespace NUnit.UiKit
 			this.selectedList.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
 				| System.Windows.Forms.AnchorStyles.Left) 
 				| System.Windows.Forms.AnchorStyles.Right)));
+			this.selectedList.ItemHeight = 16;
 			this.selectedList.Location = new System.Drawing.Point(8, 16);
 			this.selectedList.Name = "selectedList";
 			this.selectedList.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
-			this.selectedList.Size = new System.Drawing.Size(208, 95);
+			this.selectedList.Size = new System.Drawing.Size(187, 84);
 			this.selectedList.TabIndex = 0;
 			this.selectedList.DoubleClick += new System.EventHandler(this.removeCategory_Click);
 			// 
@@ -514,9 +521,9 @@ namespace NUnit.UiKit
 			// 
 			this.excludeCheckbox.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left) 
 				| System.Windows.Forms.AnchorStyles.Right)));
-			this.excludeCheckbox.Location = new System.Drawing.Point(8, 128);
+			this.excludeCheckbox.Location = new System.Drawing.Point(8, 120);
 			this.excludeCheckbox.Name = "excludeCheckbox";
-			this.excludeCheckbox.Size = new System.Drawing.Size(200, 16);
+			this.excludeCheckbox.Size = new System.Drawing.Size(179, 16);
 			this.excludeCheckbox.TabIndex = 1;
 			this.excludeCheckbox.Text = "Exclude these categories";
 			this.excludeCheckbox.CheckedChanged += new System.EventHandler(this.excludeCheckbox_CheckedChanged);
@@ -527,9 +534,9 @@ namespace NUnit.UiKit
 				| System.Windows.Forms.AnchorStyles.Left) 
 				| System.Windows.Forms.AnchorStyles.Right)));
 			this.groupBox1.Controls.Add(this.availableList);
-			this.groupBox1.Location = new System.Drawing.Point(8, 16);
+			this.groupBox1.Location = new System.Drawing.Point(8, 0);
 			this.groupBox1.Name = "groupBox1";
-			this.groupBox1.Size = new System.Drawing.Size(224, 232);
+			this.groupBox1.Size = new System.Drawing.Size(203, 272);
 			this.groupBox1.TabIndex = 0;
 			this.groupBox1.TabStop = false;
 			this.groupBox1.Text = "Available Categories";
@@ -539,10 +546,11 @@ namespace NUnit.UiKit
 			this.availableList.Anchor = ((System.Windows.Forms.AnchorStyles)((((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
 				| System.Windows.Forms.AnchorStyles.Left) 
 				| System.Windows.Forms.AnchorStyles.Right)));
+			this.availableList.ItemHeight = 16;
 			this.availableList.Location = new System.Drawing.Point(8, 24);
 			this.availableList.Name = "availableList";
 			this.availableList.SelectionMode = System.Windows.Forms.SelectionMode.MultiExtended;
-			this.availableList.Size = new System.Drawing.Size(208, 196);
+			this.availableList.Size = new System.Drawing.Size(187, 244);
 			this.availableList.TabIndex = 0;
 			this.availableList.DoubleClick += new System.EventHandler(this.addCategory_Click);
 			// 
@@ -573,9 +581,19 @@ namespace NUnit.UiKit
 		#endregion
 
 
-		public void RunTests()
+		public void RunAllTests()
 		{
-			tests.RunTests();
+			tests.RunAllTests();
+		}
+
+		public void RunSelectedTests()
+		{
+			tests.RunSelectedTests();
+		}
+
+		public void RunFailedTests()
+		{
+			tests.RunFailedTests();
 		}
 
 		private void addCategory_Click(object sender, System.EventArgs e)
@@ -589,7 +607,7 @@ namespace NUnit.UiKit
 					availableList.Items.Remove(category);
 				}
 
-				tests.SelectedCategories = this.SelectedCategories;
+				UpdateCategoryFilter();
 				if (this.SelectedCategories.Length > 0)
 					this.excludeCheckbox.Enabled = true;
 			}
@@ -606,9 +624,12 @@ namespace NUnit.UiKit
 					availableList.Items.Add(category);
 				}
 
-				tests.SelectedCategories = this.SelectedCategories;
+				UpdateCategoryFilter();
 				if (this.SelectedCategories.Length == 0)
+				{
+					this.excludeCheckbox.Checked = false;
 					this.excludeCheckbox.Enabled = false;
+				}
 			}
 		}
 
@@ -622,18 +643,18 @@ namespace NUnit.UiKit
 			tests.CheckFailedNodes();
 		}
 
-		private void tests_SelectedTestChanged(UITestNode test)
+		private void tests_SelectedTestChanged(ITest test)
 		{
 			if (SelectedTestsChanged != null) 
 			{
-			    SelectedTestsChangedEventArgs args = new SelectedTestsChangedEventArgs(test.Name, test.CountTestCases());
+				SelectedTestsChangedEventArgs args = new SelectedTestsChangedEventArgs(test.TestName.Name, test.TestCount);
 				SelectedTestsChanged(tests, args);
 			}
 		}
 
 		private void events_TestLoaded(object sender, NUnit.Util.TestEventArgs args)
 		{			
-			viewMenu.Visible = true;
+			treeMenu.Visible = true;
 
 			availableCategories = this.loader.GetCategories();
 			availableList.Items.Clear();
@@ -642,6 +663,28 @@ namespace NUnit.UiKit
 			availableList.SuspendLayout();
 			foreach (string category in availableCategories) 
 				availableList.Items.Add(category);
+
+			// tree may have restored visual state
+			if( !tests.CategoryFilter.IsEmpty )
+			{
+				ITestFilter filter = tests.CategoryFilter;
+				if ( filter is NUnit.Core.Filters.NotFilter )
+				{
+					filter = ((NUnit.Core.Filters.NotFilter)filter).BaseFilter;
+					this.excludeCheckbox.Checked = true;
+				}
+
+				foreach( string cat in ((NUnit.Core.Filters.CategoryFilter)filter).Categories )
+					if ( this.availableCategories.Contains( cat ) )
+					{
+						this.availableList.Items.Remove( cat );
+						this.selectedList.Items.Add( cat );
+						this.excludeCheckbox.Enabled = true;
+					}
+
+				UpdateCategoryFilter();
+			}
+
 			availableList.ResumeLayout();
 		}
 
@@ -670,20 +713,25 @@ namespace NUnit.UiKit
 			availableList.ResumeLayout();
 
 			// Tell the tree what is selected
-			tests.SelectedCategories = this.SelectedCategories;
+			UpdateCategoryFilter();
 		}
 
 		private void excludeCheckbox_CheckedChanged(object sender, System.EventArgs e)
 		{
-			tests.ExcludeSelectedCategories = excludeCheckbox.Checked;
+			UpdateCategoryFilter();
 		}
 
-		private void Events_TestUnloaded(object sender, NUnit.Util.TestEventArgs args)
+		private void events_TestUnloaded(object sender, NUnit.Util.TestEventArgs args)
 		{
-			viewMenu.Visible = false;
+			availableCategories.Clear();
+			availableList.Items.Clear();
+			selectedList.Items.Clear();
+			excludeCheckbox.Checked = false;
+			excludeCheckbox.Enabled = false;
+			treeMenu.Visible = false;
 		}
 
-		private void tests_CheckedTestChanged(UITestNode[] tests)
+		private void tests_CheckedTestChanged(ITest[] tests)
 		{
 			if (SelectedTestsChanged != null) 
 			{
@@ -698,12 +746,37 @@ namespace NUnit.UiKit
 
 		private void checkBoxesMenuItem_Click(object sender, System.EventArgs e)
 		{
-			ShowCheckBoxes = !checkBoxesMenuItem.Checked;
+			Services.UserSettings.SaveSetting( "Options.ShowCheckBoxes", 
+				ShowCheckBoxes = !checkBoxesMenuItem.Checked );
 			
 			// Temporary till we can save tree state and restore
 			//this.SetInitialExpansion();
 		}
 
+		private void UpdateCategoryFilter()
+		{
+			TestFilter catFilter;
+
+			if ( SelectedCategories == null || SelectedCategories.Length == 0 )
+				catFilter = TestFilter.Empty;
+			else
+				catFilter = new NUnit.Core.Filters.CategoryFilter( SelectedCategories );
+			if ( excludeCheckbox.Checked )
+				catFilter = new NUnit.Core.Filters.NotFilter( catFilter );
+
+			tests.CategoryFilter = catFilter;
+		}
+
+		private void tests_CheckBoxesChanged(object sender, System.EventArgs e)
+		{
+			ShowCheckBoxes = tests.CheckBoxes;
+		}
+
+		private void UserSettings_Changed(object sender, SettingsEventArgs args)
+		{
+			if ( args.SettingName == "Options.ShowCheckBoxes" )
+				this.ShowCheckBoxes = Services.UserSettings.GetSetting( args.SettingName, false );
+		}
 	}
 
 	public class SelectedTestsChangedEventArgs : EventArgs 
