@@ -1,5 +1,5 @@
 using System;
 using System.Reflection;
 
-[assembly: AssemblyTitle("NUnit-Gui")]
+[assembly: AssemblyTitle("NUnit")]
 [assembly: AssemblyDescription("")]
\ No newline at end of file
