@@ -1,31 +1,8 @@
-#region Copyright (c) 2003, James W. Newkirk, Michael C. Two, Alexei A. Vorontsov, Charlie Poole, Philip A. Craig
-/************************************************************************************
-'
-' Copyright  2002-2003 James W. Newkirk, Michael C. Two, Alexei A. Vorontsov, Charlie Poole
-' Copyright  2000-2002 Philip A. Craig
-'
-' This software is provided 'as-is', without any express or implied warranty. In no 
-' event will the authors be held liable for any damages arising from the use of this 
-' software.
-' 
-' Permission is granted to anyone to use this software for any purpose, including 
-' commercial applications, and to alter it and redistribute it freely, subject to the 
-' following restrictions:
-'
-' 1. The origin of this software must not be misrepresented; you must not claim that 
-' you wrote the original software. If you use this software in a product, an 
-' acknowledgment (see the following) in the product documentation is required.
-'
-' Portions Copyright  2002-2003 James W. Newkirk, Michael C. Two, Alexei A. Vorontsov, Charlie Poole
-' or Copyright  2000-2002 Philip A. Craig
-'
-' 2. Altered source versions must be plainly marked as such, and must not be 
-' misrepresented as being the original software.
-'
-' 3. This notice may not be removed or altered from any source distribution.
-'
-'***********************************************************************************/
-#endregion
+// ****************************************************************
+// This is free software licensed under the NUnit license. You
+// may obtain a copy of the license as well as information regarding
+// copyright ownership at http://nunit.org/?p=license&r=2.4.
+// ****************************************************************
 
 namespace NUnit.Framework.Tests
 {
@@ -70,21 +47,6 @@ namespace NUnit.Framework.Tests
 				Assert.AreEqual( true,  InputsAreStrings( "some", "value" ) );
 			}
 
-			/// <summary>
-			/// 
-			/// </summary>
-			/// 
-			[Test]
-			public void TestCreateStringBuilder()
-			{
-				Assert.AreEqual( "",        CreateStringBuilder( null ).ToString() );
-				Assert.AreEqual( "",        CreateStringBuilder( "" ).ToString() );
-				Assert.AreEqual( "a",       CreateStringBuilder( "a" ).ToString() );
-				Assert.AreEqual( "message", CreateStringBuilder( "message" ).ToString() );
-				Assert.AreEqual( "message 5 from me", 
-					CreateStringBuilder( "message {0} from {1}", 5, "me" ).ToString() );
-			}
-
 			/// <summary>
 			/// Tests the string-clipping method to see if it clips at the
 			/// expected positions.
@@ -187,6 +149,8 @@ namespace NUnit.Framework.Tests
                 
 				Assert.AreEqual( "\\t", ConvertWhitespace( "\t" ) );
 				Assert.AreEqual( "\\t\\n", ConvertWhitespace( "\t\n" ) );
+
+				Assert.AreEqual( "\\\\r\\\\n", ConvertWhitespace( "\\r\\n" ) );
 			}
 
 			/// <summary>
@@ -196,84 +160,83 @@ namespace NUnit.Framework.Tests
 			[Test]
 			public void TestFormatMessageForFailNotEquals()
 			{
-				Assert.AreEqual( @"
-	expected: <(null)>
-	 but was: <(null)>", 
+				Assert.AreEqual( "	expected: <(null)>" + Environment.NewLine + 
+					"	 but was: <(null)>" + Environment.NewLine, 
 					GetMsg( null, null, null ) );
 
-				Assert.AreEqual( @"
-	expected: <(null)>
-	 but was: <(null)>", 
+				Assert.AreEqual( 
+					"	expected: <(null)>" + Environment.NewLine + 
+					"	 but was: <(null)>" + Environment.NewLine, 
 					GetMsg(null, null,  "") );
 
-				Assert.AreEqual( @"message
-	expected: <(null)>
-	 but was: <(null)>", 
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	expected: <(null)>" +
+					Environment.NewLine + "	 but was: <(null)>" + Environment.NewLine, 
 					GetMsg(null, null, "message" ) );
 
-				Assert.AreEqual( @"message
-	expected: <1>
-	 but was: <2>", 
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	expected: <1>" +
+					Environment.NewLine + "	 but was: <2>" + Environment.NewLine, 
 					GetMsg( 1, 2, "message" ) );
 
-				Assert.AreEqual( @"message
-	expected: <"""">
-	 but was: <(null)>", 
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	expected: <\"\">" +
+					Environment.NewLine + "	 but was: <(null)>" + Environment.NewLine, 
 					GetMsg( "", null, "message" ) );
 
-				Assert.AreEqual( @"message
-	expected: <1>
-	 but was: <""1"">",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	expected: <1>" +
+					Environment.NewLine + "	 but was: <\"1\">" + Environment.NewLine,
 					GetMsg( 1, "1", "message" ) );
 
-				Assert.AreEqual( @"message 5
-	expected: <1>
-	 but was: <""1"">",
+				Assert.AreEqual( "message 5" +
+					Environment.NewLine + "	expected: <1>" +
+					Environment.NewLine + "	 but was: <\"1\">" + Environment.NewLine,
 					GetMsg( 1, "1", "message {0}", 5 ) );
 			
-				Assert.AreEqual( @"message
-	String lengths differ.  Expected length=1, but was length=2.
-	Strings differ at index 1.
-	expected: <""a"">
-	 but was: <""aa"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths differ.  Expected length=1, but was length=2." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"a\">" +
+					Environment.NewLine + "	 but was: <\"aa\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					GetMsg( "a", "aa", "message" ) );
 
-				Assert.AreEqual( @"message
-	String lengths are both 2.
-	Strings differ at index 1.
-	expected: <""aa"">
-	 but was: <""ab"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths are both 2." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"aa\">" +
+					Environment.NewLine + "	 but was: <\"ab\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					GetMsg( "aa", "ab", "message" ) );
 
-				Assert.AreEqual( @"message
-	String lengths differ.  Expected length=1, but was length=3.
-	Strings differ at index 1.
-	expected: <""a"">
-	 but was: <""abc"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths differ.  Expected length=1, but was length=3." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"a\">" +
+					Environment.NewLine + "	 but was: <\"abc\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					GetMsg( "a", "abc", "message" ) );
 
-				Assert.AreEqual( @"message
-	String lengths are both 3.
-	Strings differ at index 1.
-	expected: <""123"">
-	 but was: <""1x3"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths are both 3." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"123\">" +
+					Environment.NewLine + "	 but was: <\"1x3\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					GetMsg( "123", "1x3", "message" ) );
 
-				Assert.AreEqual( @"message
-	String lengths differ.  Expected length=5, but was length=3.
-	Strings differ at index 3.
-	expected: <""12345"">
-	 but was: <""123"">
-	---------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths differ.  Expected length=5, but was length=3." +
+					Environment.NewLine + "	Strings differ at index 3." +
+					Environment.NewLine + "	expected: <\"12345\">" +
+					Environment.NewLine + "	 but was: <\"123\">" +
+					Environment.NewLine + "	---------------^" + Environment.NewLine,
 					GetMsg( "12345", "123", "message" ) );
 
-				Assert.AreEqual( @"message
-	expected: <" + new DateTime(2005, 6, 1, 0, 0, 0) + @">
-	 but was: <" + new DateTime(2005, 6, 7, 0, 0, 0) + ">",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	expected: <" + new DateTime(2005, 6, 1, 0, 0, 0) + ">" +
+					Environment.NewLine + "	 but was: <" + new DateTime(2005, 6, 7, 0, 0, 0) + ">" + Environment.NewLine,
 					GetMsg( new DateTime( 2005, 6, 1 ), new DateTime( 2005, 6, 7 ), "message" ) );
 			}
 
@@ -283,12 +246,12 @@ namespace NUnit.Framework.Tests
 				AssertionFailureMessage msg = new AssertionFailureMessage( "message" );
 				msg.DisplayDifferences( "Name", "NAMES", true );
 
-				Assert.AreEqual( @"message
-	String lengths differ.  Expected length=4, but was length=5.
-	Strings differ at index 4.
-	expected: <""Name"">
-	 but was: <""NAMES"">
-	----------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths differ.  Expected length=4, but was length=5." +
+					Environment.NewLine + "	Strings differ at index 4." +
+					Environment.NewLine + "	expected: <\"Name\">" +
+					Environment.NewLine + "	 but was: <\"NAMES\">" +
+					Environment.NewLine + "	----------------^" + Environment.NewLine,
 					msg.ToString() );
 			}
 
@@ -310,30 +273,30 @@ namespace NUnit.Framework.Tests
 			/// where the differences actually are.
 			/// </summary>
 			[Test]
-			public void TestFormatMessageForFailNotEqualsNewlines()
+			public void TestFormatMessageForFailNotEqualsNewlines2()
 			{
-				Assert.AreEqual( @"message
-	String lengths differ.  Expected length=2, but was length=3.
-	Strings differ at index 1.
-	expected: <""a\r"">
-	 but was: <""aa\r"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths differ.  Expected length=2, but was length=3." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"a\\r\">" +
+					Environment.NewLine + "	 but was: <\"aa\\r\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					GetMsg( "a\r", "aa\r", "message" ) );
 
-				Assert.AreEqual( @"message
-	String lengths differ.  Expected length=2, but was length=3.
-	Strings differ at index 1.
-	expected: <""a\n"">
-	 but was: <""aa\n"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths differ.  Expected length=2, but was length=3." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"a\\n\">" +
+					Environment.NewLine + "	 but was: <\"aa\\n\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					GetMsg( "a\n", "aa\n", "message" ) );
 
-				Assert.AreEqual( @"message
-	String lengths are both 6.
-	Strings differ at index 5.
-	expected: <""aa\r\naa"">
-	 but was: <""aa\r\nab"">
-	-------------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "	String lengths are both 6." +
+					Environment.NewLine + "	Strings differ at index 5." +
+					Environment.NewLine + "	expected: <\"aa\\r\\naa\">" +
+					Environment.NewLine + "	 but was: <\"aa\\r\\nab\">" +
+					Environment.NewLine + "	-------------------^" + Environment.NewLine,
 					GetMsg( "aa\r\naa", "aa\r\nab", "message" ) );
 			}
 
@@ -370,12 +333,12 @@ namespace NUnit.Framework.Tests
 
 				string sFirst  = "00000000001111111111222222222233333333334444444444555555555566666666667777777777888888888899999999990000000000111111111122222222223333333333444444444455555555556666";
 				string sSecond = "00000000001111111111222222222233333333334444444444555555555566666666667777777777888888888899999999990000000000111111111122222222223333333333444444444455555555556666++";
-				Assert.AreEqual( @"
-	String lengths differ.  Expected length=164, but was length=166.
-	Strings differ at index 164.
-	expected: <""...23333333333444444444455555555556666"">
-	 but was: <""...23333333333444444444455555555556666++"">
-	--------------------------------------------------^",
+				Assert.AreEqual(
+"	String lengths differ.  Expected length=164, but was length=166." + System.Environment.NewLine +
+"	Strings differ at index 164." + System.Environment.NewLine + 
+"	expected: <\"...23333333333444444444455555555556666\">" + System.Environment.NewLine + 
+"	 but was: <\"...23333333333444444444455555555556666++\">" + System.Environment.NewLine + 
+"	--------------------------------------------------^" + Environment.NewLine,
 					GetMsg( sFirst, sSecond, null ) );
 			}
 
@@ -389,12 +352,12 @@ namespace NUnit.Framework.Tests
 			{
 				string sFirst  = "0000000000111111111122";
 				string sSecond = "0000000000111111111122++";
-				Assert.AreEqual( @"
-	String lengths differ.  Expected length=22, but was length=24.
-	Strings differ at index 22.
-	expected: <""0000000000111111111122"">
-	 but was: <""0000000000111111111122++"">
-	----------------------------------^",
+				Assert.AreEqual( 
+					"	String lengths differ.  Expected length=22, but was length=24." + 
+					Environment.NewLine + "	Strings differ at index 22." + 
+					Environment.NewLine + "	expected: <\"0000000000111111111122\">" + 
+					Environment.NewLine + "	 but was: <\"0000000000111111111122++\">" + 
+					Environment.NewLine + "	----------------------------------^" + Environment.NewLine,
 					GetMsg( sFirst, sSecond, null ) );
 			}
 
@@ -425,7 +388,12 @@ namespace NUnit.Framework.Tests
 				{
 					string sFirst  = new string( '=', i );
 					string sSecond = new string( '=', i ) + sExtra;
-					Assert.AreEqual( "\r\n\tString lengths differ.  Expected length=" + i + ", but was length=" + (i+sExtra.Length) + ".\r\n\tStrings differ at index "+ i +".\r\n\texpected: <\""+ sFirst +"\">\r\n\t but was: <\""+ sSecond +"\">\r\n\t" + (new string('-',actualPrefix.Length+i+3)) + "^",
+					Assert.AreEqual( 
+						"\tString lengths differ.  Expected length=" + i + ", but was length=" + (i+sExtra.Length) + "." + System.Environment.NewLine + 
+						"\tStrings differ at index "+ i +"." + System.Environment.NewLine + 
+						"\texpected: <\""+ sFirst +"\">" + System.Environment.NewLine + 
+						"\t but was: <\""+ sSecond +"\">" + System.Environment.NewLine + 
+						"\t" + (new string('-',actualPrefix.Length+i+3)) + "^" + Environment.NewLine,
 						GetMsg( sFirst, sSecond, null ),
 						"Failed at index " + i);
 				}
@@ -441,7 +409,12 @@ namespace NUnit.Framework.Tests
 				{
 					string sFirst  = new string( '=', i );
 					string sSecond = new string( '=', i ) + sExtra;
-					Assert.AreEqual( "\r\n\tString lengths differ.  Expected length=" + i + ", but was length=" + (i+sExtra.Length) + ".\r\n\tStrings differ at index "+ i +".\r\n\texpected: <\""+ sExpected +"\">\r\n\t but was: <\""+ sActual +"\">\r\n\t" + (new string('-',actualPrefix.Length+"...".Length+PreClipLength+3)) + "^",
+					Assert.AreEqual( 
+						"\tString lengths differ.  Expected length=" + i + ", but was length=" + (i+sExtra.Length) + "." + System.Environment.NewLine + 
+						"\tStrings differ at index "+ i +"." + System.Environment.NewLine + 
+						"\texpected: <\""+ sExpected +"\">" + System.Environment.NewLine + 
+						"\t but was: <\""+ sActual +"\">" + System.Environment.NewLine + 
+						"\t" + (new string('-',actualPrefix.Length+"...".Length+PreClipLength+3)) + "^" + Environment.NewLine,
 						GetMsg( sFirst, sSecond, null ),
 						"Failed at index " + i );
 				}
@@ -454,33 +427,45 @@ namespace NUnit.Framework.Tests
 
 				msg = new AssertionFailureMessage( "message");
 				msg.DisplayListElements( "label:", new object[] { "a", "b", "c" }, 0, 3 );
-				Assert.AreEqual( "message\r\nlabel:<<\"a\">,<\"b\">,<\"c\">>",
+				Assert.AreEqual( 
+					"message" + System.Environment.NewLine + 
+					"label:<<\"a\">,<\"b\">,<\"c\">>" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message");
 				msg.DisplayListElements( "label:", new object[] { "a", "b", "c" }, 0, 5 );
-				Assert.AreEqual( "message\r\nlabel:<<\"a\">,<\"b\">,<\"c\">>",
+				Assert.AreEqual( 
+					"message" + System.Environment.NewLine + 
+					"label:<<\"a\">,<\"b\">,<\"c\">>" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message");
 				msg.DisplayListElements( "label:", new object[] { "a", "b", "c" }, 1, 1 );
-				Assert.AreEqual( "message\r\nlabel:<<\"b\">,...>",
+				Assert.AreEqual( 
+					"message" + System.Environment.NewLine + 
+					"label:<<\"b\">,...>" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message");
 				msg.DisplayListElements( "label:", new object[] { "a", "b", "c" }, 1, 5 );
-				Assert.AreEqual( "message\r\nlabel:<<\"b\">,<\"c\">>",
+				Assert.AreEqual( 
+					"message" + System.Environment.NewLine + 
+					"label:<<\"b\">,<\"c\">>" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message");
 				msg.DisplayListElements( "label:", new object[0], 0, 5 );
-				Assert.AreEqual( "message\r\nlabel:<empty>",
+				Assert.AreEqual( 
+					"message" + System.Environment.NewLine + 
+					"label:<empty>" + Environment.NewLine,
 					msg.ToString() );
 
 
 				msg = new AssertionFailureMessage( "message");
 				msg.DisplayListElements( "label:", null, 0, 5 );
-				Assert.AreEqual( "message\r\nlabel:<null>",
+				Assert.AreEqual( 
+					"message" + System.Environment.NewLine + 
+					"label:<null>" + Environment.NewLine,
 					msg.ToString() );
 			}
 
@@ -495,10 +480,10 @@ namespace NUnit.Framework.Tests
 					new object[] { "one", "two", "three", "four", "five" },
 					3 );
 
-				Assert.AreEqual( @"message
-Array lengths differ.  Expected length=3, but was length=5.
-Arrays differ at index 3.
-   extra:<<""four"">,<""five"">>",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "Array lengths differ.  Expected length=3, but was length=5." +
+					Environment.NewLine + "Arrays differ at index 3." +
+					Environment.NewLine + "   extra:<<\"four\">,<\"five\">>" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message" );
@@ -507,10 +492,10 @@ Arrays differ at index 3.
 					new object[] { "one", "two", "three" },
 					3 );
 
-				Assert.AreEqual( @"message
-Array lengths differ.  Expected length=5, but was length=3.
-Arrays differ at index 3.
- missing:<<""four"">,<""five"">>",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "Array lengths differ.  Expected length=5, but was length=3." +
+					Environment.NewLine + "Arrays differ at index 3." +
+					Environment.NewLine + " missing:<<\"four\">,<\"five\">>" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message" );
@@ -519,14 +504,14 @@ Arrays differ at index 3.
 					new object[] { "one", "two", "ten" },
 					2 );
 
-				Assert.AreEqual( @"message
-Array lengths are both 3.
-Arrays differ at index 2.
-	String lengths differ.  Expected length=5, but was length=3.
-	Strings differ at index 1.
-	expected: <""three"">
-	 but was: <""ten"">
-	-------------^",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "Array lengths are both 3." +
+					Environment.NewLine + "Arrays differ at index 2." +
+					Environment.NewLine + "	String lengths differ.  Expected length=5, but was length=3." +
+					Environment.NewLine + "	Strings differ at index 1." +
+					Environment.NewLine + "	expected: <\"three\">" +
+					Environment.NewLine + "	 but was: <\"ten\">" +
+					Environment.NewLine + "	-------------^" + Environment.NewLine,
 					msg.ToString() );
 
 				msg = new AssertionFailureMessage( "message" );
@@ -535,11 +520,24 @@ Arrays differ at index 2.
 					new object[] { 1, 2, 10 },
 					2 );
 
-				Assert.AreEqual( @"message
-Array lengths are both 3.
-Arrays differ at index 2.
-	expected: <3>
-	 but was: <10>",
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "Array lengths are both 3." +
+					Environment.NewLine + "Arrays differ at index 2." +
+					Environment.NewLine + "	expected: <3>" +
+					Environment.NewLine + "	 but was: <10>" + Environment.NewLine,
+					msg.ToString() );
+
+				msg = new AssertionFailureMessage( "message" );
+				msg.DisplayArrayDifferences( 
+					new object[,] { { 1, 2, 3 }, { 4, 5, 6 } },
+					new object[,] { { 1, 2, 9 }, { 4, 5, 6 } },
+					2 );
+
+				Assert.AreEqual( "message" +
+					Environment.NewLine + "Array lengths are both 6." +
+					Environment.NewLine + "Arrays differ at index 2." +
+					Environment.NewLine + "	expected: <3>" +
+					Environment.NewLine + "	 but was: <9>" + Environment.NewLine,
 					msg.ToString() );
 
 			}
